// Generated by dts-bundle v0.7.3
// Dependencies for this module:
//   ../atma-server
//   ../atma-server/HttpApplication/LifecycleEvents
//   ../@slack/web-api
//   ../atma-io

declare module 'atma-server-monit' {
    import { Application } from 'atma-server';
    import { IMonitOptions } from 'atma-server-monit/MonitWorker';
    import { ILoggerOpts, LoggerFile, ILogger } from 'atma-server-monit/fs/LoggerFile';
    export namespace Monit {
        function startLogger(opts: IMonitOptions): Promise<void>;
        function start(app: Application, opts: IMonitOptions): void;
        function createChannel(name: string, opts?: Partial<ILoggerOpts>): ILogger;
        function createChannelReader(channel: LoggerFile): any;
        function createChannelReader(name: string, opts?: Partial<ILoggerOpts>): any;
        function flush(): void;
        function error(error: Error): void;
    }
}

declare module 'atma-server-monit/MonitWorker' {
    import { LifecycleEvent, LifecycleEvents } from 'atma-server/HttpApplication/LifecycleEvents';
    import { SlackClient } from 'atma-server-monit/Slack';
    import { LoggerFile, ILoggerOpts } from 'atma-server-monit/fs/LoggerFile';
    import { ChannelReader } from 'atma-server-monit/reader/ChannelReader';
    export interface IMonitOptions {
        directory?: string;
        channels?: {
            [name: string]: ILoggerOpts;
        };
        slack?: {
            token: string;
            channelId: string;
        };
        filterForSlack?: (event: LifecycleEvent) => boolean;
    }
    export class MonitWorker {
        events: LifecycleEvents;
        opts: IMonitOptions & {
            disableDefaultLoggers?: boolean;
        };
        slack: SlackClient;
        loggers: {
            start?: LoggerFile;
            requests?: LoggerFile;
            errors?: LoggerFile;
            [name: string]: LoggerFile;
        };
        constructor(events: LifecycleEvents, opts: IMonitOptions & {
            disableDefaultLoggers?: boolean;
        });
        createChannel(name: string, opts?: Partial<ILoggerOpts>): LoggerFile;
        createChannelReader(channel: LoggerFile): ChannelReader;
        watch(events: LifecycleEvents): void;
        writeError(error: Error): void;
        /** Flush all buffered content to disk */
        flush(): void;
        restoreChannelsAsync(): Promise<void>;
    }
}

declare module 'atma-server-monit/fs/LoggerFile' {
    import { ICsvColumn } from 'atma-server-monit/model/ICsvColumn';
    export interface ILoggerOpts {
        directory: string;
        fileCountMax?: number;
        fileBytesMax?: number;
        fileMessagesMax?: number;
        messageBufferMax?: number;
        writeTimeout?: number;
        fields?: ICsvColumn[];
        addCsvHeader?: boolean;
        columns?: ICsvColumn[];
    }
    export interface ILogger {
        writeRow(cells: any[]): any;
        write(mix: string | any[]): void;
        flush(): any;
        removeAll(): Promise<any>;
    }
    export class EmptyLoggerFile implements ILogger {
        writeRow(cells: any[]): void;
        write(mix: string | any[]): void;
        flush(): void;
        removeAll(): any;
    }
    export class LoggerFile implements ILogger {
        directory: string;
        opts: ILoggerOpts;
        static create(key: string, opts: ILoggerOpts): LoggerFile;
        static prepair(opts: ILoggerOpts): LoggerFile;
        static restore(directory: string, key: string, options?: ILoggerOpts): Promise<LoggerFile>;
        protected constructor(opts: ILoggerOpts);
        writeRow(cells: any[]): void;
        writeRows(cellsMany: any[][]): void;
        write(mix: string | any[]): void;
        get path(): string;
        flush(): void;
        removeAll(): Promise<any>;
        protected initOptions(opts: ILoggerOpts): void;
        protected init(): void;
    }
}

declare module 'atma-server-monit/Slack' {
    import { WebClient } from '@slack/web-api';
    export class SlackClient {
        access_token: string;
        team_id: string;
        enterprise_id: string;
        web: WebClient;
        token: string;
        channelId: string;
        constructor(opts: {
            token: string;
            channelId: string;
        });
        login(): Promise<void>;
        send(message: string): Promise<void>;
    }
}

declare module 'atma-server-monit/reader/ChannelReader' {
    import { FileReader } from 'atma-server-monit/reader/FileReader'; 
     import { LoggerFile } from 'atma-server-monit/fs/LoggerFile';
    import { ICsvColumn } from 'atma-server-monit/model/ICsvColumn';
    import { GetChannelParams } from 'atma-server-monit/reader/LogsReader';
    export interface IChannelLinesQuery {
            offset?: number;
            limit?: number;
            from?: Date;
            to?: Date;
    }
    export class ChannelReader {
            channel: LoggerFile;
            constructor(channel: LoggerFile);
            fetch(query: IChannelLinesQuery): Promise<{
                    rows: any[];
            }>;
            getDays(): Promise<{
                    day: string;
            }[]>;
            getData(query: GetChannelParams): Promise<{
                    columns: ICsvColumn[];
                    rows: any[][];
                    size: number;
            }>;
            protected getReaders(): Promise<FileReader[]>;
    }
}

declare module 'atma-server-monit/model/ICsvColumn' {
    export interface ICsvColumn {
        idx?: number;
        name: string;
        type?: 'string' | 'number' | 'date' | 'text' | 'boolean';
        summable?: boolean;
        groupable?: boolean;
        sortable?: boolean;
        filterable?: boolean;
    }
}

declare module 'atma-server-monit/reader/FileReader' {
    import { File } from 'atma-io';
    import { DayDate } from 'atma-server-monit/model/DayDate';
    import { LoggerFile } from 'atma-server-monit/fs/LoggerFile';
    import { ICsvColumn } from "atma-server-monit/model/ICsvColumn";
    type FileType = InstanceType<typeof File>;
    export interface IFileQuery {
        offset?: number;
        limit?: number;
        from?: Date;
        to?: Date;
    }
    export class FileReader {
        channel: LoggerFile;
        uri: string;
        idxFile?: FileType;
        day: DayDate;
        nr: number;
        fields: ICsvColumn[];
        cached: boolean;
        table: any[][];
        hasHeader: boolean;
        static create(channel: LoggerFile, uri: string, idxFile?: FileType): FileReader;
        protected constructor(channel: LoggerFile, uri: string, idxFile?: FileType);
        read(): Promise<any[][]>;
        fetch(opts: IFileQuery): Promise<{
            total: number;
            rows: any[][];
        }>;
    }
    export namespace Csv {
        function parseType(val: string, field: ICsvColumn): string | number | Date;
        function splitRow(row: string): any[];
    }
    export {};
}

declare module 'atma-server-monit/reader/LogsReader' {
    import { ICsvColumn } from 'atma-server-monit/model/ICsvColumn'; 
     import { MonitWorker } from 'atma-server-monit/MonitWorker';
    import { ITableColumnFilter } from 'atma-server-monit/model/Table';
    import { DayDate } from 'atma-server-monit/model/DayDate';
    export class LogsReader {
            monit: MonitWorker;
            constructor(monit: MonitWorker);
            getChannels(): {
                    name: string;
                    directory: string;
                    columns: ICsvColumn[];
            }[];
            getChannelInfo(key: string): {
                    name: string;
                    directory: string;
                    columns: ICsvColumn[];
            };
            getChannelDays(key: string): Promise<{
                    day: string;
            }[]>;
            getChannelData(query: GetChannelParams): Promise<{
                    columns: ICsvColumn[];
                    rows: any[][];
                    size: number;
            }>;
    }
    export class GetChannelParams {
            key: string;
            sortByColumnIdx?: number;
            sortDir?: 'asc' | 'desc';
            columnFilters?: ITableColumnFilter[];
            day?: DayDate;
            rangeStart?: Date;
            rangeEnd?: Date;
            offset?: number;
            limit?: number;
    }
}

declare module 'atma-server-monit/model/DayDate' {
    export class DayDate implements IDay {
        year: number;
        date: number;
        month: number;
        constructor(year: number, month: number, date: number);
        constructor(str: string);
        constructor(day: IDay);
        constructor(date: Date);
        toDate(): Date;
        isEqual(date: DayDate): boolean;
        isSame(date: Date): boolean;
        isAfter(date: Date): boolean;
        isBefore(date: Date): boolean;
        valueOf(): number;
        format(pattern: any): any;
        serialize(): string;
        getFullYear(): number;
        getMonth(): number;
        getDate(): number;
        getHours(): number;
        getMinutes(): number;
        getSeconds(): number;
        static now(): DayDate;
    }
    export interface IDay {
        year: number;
        date: number;
        month: number;
    }
}

declare module 'atma-server-monit/model/Table' {
    import { ICsvColumn } from 'atma-server-monit/model/ICsvColumn';
    export interface ITableQuery {
        sortByColumnIdx?: number;
        sortDir?: 'asc' | 'desc';
        columnFilters?: ITableColumnFilter[];
        rangeStart?: Date;
        rangeEnd?: Date;
        offset?: number;
        limit?: number;
    }
    export interface ITableColumnFilter {
        columnIdx: number;
        q: string;
    }
    export class Table {
        rows: any[][];
        dateIdx: number;
        size: number;
        constructor(fields: ICsvColumn[], rows: any[][]);
        getTable(params: ITableQuery): any[][];
    }
}

